# Generated by Django 2.2.8 on 2022-05-11 09:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('departments', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='department',
            name='charger',
            field=models.IntegerField(null=True, verbose_name='负责人ID'),
        ),
    ]
