# Generated by Django 2.2.8 on 2022-05-12 11:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('departments', '0005_auto_20220512_1138'),
    ]

    operations = [
        migrations.RenameField(
            model_name='department',
            old_name='charger',
            new_name='charger_id',
        ),
    ]
